/*
 ************************************************************************************************
 *                                                                                              *
 *  OSA cooperative RTOS for microcontrollers PIC (Microchip) and 8-bit AVR (Atmel)
 *                                                                                              *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro                              *
 *                  http://picosa.narod.ru                                                      *
 *                                                                                              *
 *----------------------------------------------------------------------------------------------*
 *                                                                                              *
 *  File:           osa_winavr.c                                                                *    
 *                                                                                              *
 *  Compilers:      WinAVR                                                                      *
 *                                                                                              *
 *  Programmer:     Timofeev Victor                                                             *
 *                  osa@pic24.ru, testerplus@mail.ru                                            *
 *                                                                                              *
 *  Description:    OSA functions for WinAVR                                                    *
 *                                                                                              *
 *  History:        07.03.2010 -    File created
 * 
 *                  31.05.2010 -    1. Removed limitation for WinAVR: any optimization level
 *                                     can be used now
 *
 ************************************************************************************************
 */

#define OS_Init_ProcSpec()      // Empty macro for WinAVR


#if XRAMEND < 0x100
    __osa_uint8  _OS_SP_Temp;
#else
    __osa_uint16 _OS_SP_Temp;
#endif



/************************************************************************************************
 *                                                                                              *
 *                           K E R N E L   F U N C T I O N S                                    *
 *                                                                                              *
 ************************************************************************************************/

void  _OS_SET_FSR_CUR_TASK (void)     
{
    asm(" lds r26, _OS_CurTask "); 
    asm(" lds r27, _OS_CurTask+1 ");
    asm("ret");
}

//******************************************************************************

void __OS_JumpToTask (void)
{
    _OS_Store_SP();
    _OS_SET_FSR_CUR_TASK();
    _OS_SetPC();
    _OS_Restore_Temp();    /*100531.1*/
    asm(" ret ");
}



/*
 ********************************************************************************
 *                                                                              *
 *  void OS_SchedRetPoint (void)                                                *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Return to sheduler with saving current task return point    *
 *                  Called througth "jmp" or "rjmp"                             *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

void OS_SchedRetPoint (void)
{
    _OS_SET_FSR_CUR_TASK();
    _OS_SavePC();
    _OS_bTaskReady = 1;
    __OS_JMP("OS_SchedRetPointNoSave");
}



/*
 ********************************************************************************
 *                                                                              *
 *  void OS_SchedRetPointNoSave (void)                                          *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Return to scheduler without saving context                  *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

void OS_SchedRetPointNoSave (void)
{
    _OS_Save_Temp();    /*100531.1*/
    _OS_Restore_SP();
    asm(" ret ");
}


/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClearReady (void)                                                  *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Clear bReady flag and return to the sheduler with seaving   *
 *                  task return point                                           *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 ********************************************************************************
 */

void OS_ClearReady (void)
{
    asm("   ldi     r24, 0    ");
    __OS_JMP("_OS_ClrReadySetClrCanContinue");
}


/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClearReadySetCanContinue (void)                                    *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Clear bReady flag, set bCanContinue flag and return to the  *
 *                  sheduler with seaving task return point                     *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 ********************************************************************************
 */

void OS_ClearReadySetCanContinue (void)
{
    asm("   ldi    r24, 1     ");
    __OS_JMP("_OS_ClrReadySetClrCanContinue");
}


/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClrReadySetClrCanContinue (void)                                   *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Addition function. Called only by GOTO from:                *
 *                      OS_ClearReady()                                         *
 *                      OS_ClearReadySetCanContinue()                           *
 *                  Stack has pointer of task who called theese services.       *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */


void _OS_ClrReadySetClrCanContinue (void)
{
    asm("   lds     r22, OS_state       ");
    asm("   cbr     r22, (1<<3)         ");     // _OS_bTaskReady = 0

    #if defined(OS_ENABLE_TTIMERS)
        asm("   sbrs    r22, 4          ");     // if (!_OS_bTaskDelay) _OS_bTaskCanContinue = 0; 
        asm("   cbr     r22, (1<<5)     ");
        asm("   sbrc    r24, 0          ");     // if (_OS_temp & 0x01) _OS_bTaskCanContinue = 1;
        asm("   sbr     r22, (1<<5)     ");
    #endif

    asm("   sts     OS_state, r22       ");

    OS_Flags.bEventError = 0;

    _OS_SET_FSR_CUR_TASK();
    _OS_SavePC();
    _OS_Push_Back();
    asm(" ret ");
}



















/************************************************************************************************
 *                                                                                              *
 *                            S Y S T E M   F U N C T I O N S                                   *
 *                                                                                              *
 ************************************************************************************************/







/*
 ********************************************************************************
 *                                                                              *
 *  char OS_DI (void)                                                           *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Disable interrupt with GIE saving                           *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      __osa_word - current SREG value                             *
 *                                                                              *
 ********************************************************************************
 */

//-------------------------------------------------------------------------------
#define OS_DI_DEFINED
//-------------------------------------------------------------------------------

__osa_word OS_DI (void)
{
    char temp;
    temp = SREG;
    asm("cli");
    return temp; 
}

void _OS_DI_INT (void)
{
    asm("   in      r22, 0x3F       ");
    asm("   sts     _OS_temp_I, r22 ");
    asm("   cli                     ");
    asm("   ret                     ");
}


/*
 ********************************************************************************
 *                                                                              *
 *  char OS_RI (void)                                                           *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Restore saved by OS_DI() GIEx states                        *
 *                                                                              *
 *  parameters:     char - bit 7 = previos GIE value, bits 0..6 = 0             *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

//-------------------------------------------------------------------------------
#define OS_RI_DEFINED
//-------------------------------------------------------------------------------

void OS_RI (__osa_word temp)
{
    SREG = temp;
}

void _OS_RI_INT (void)
{
    asm("   lds     r22, _OS_temp_I ");
    asm("   out     0x3F, r22       ");
    asm("   ret                     ");
}





/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_CheckEvent (char bEvent)                                           *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel througth         *
 *                  waiting services)                                           *
 *                                                                              *
 *                  Check condition of bEvent. Accordint to bEvent value and    *
 *                  current state of bReady, bDelay and bCanContinue flags,     *
 *                  task becames ready to execute or still remains in waiting   *
 *                  mode.                                                       *
 *                                                                              *
 *  parameters:     bEvent - zero, or non-zero condition                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define _OS_CheckEvent_DEFINED
//------------------------------------------------------------------------------



void _OS_CheckEvent (__osa_word bEvent)
{
    //------------------------------------------------------------------------------
    #ifdef OS_DISABLE_PRIORITY
    //------------------------------------------------------------------------------

        asm("   lds     r22, OS_Flags   ");     // OS_Flags.bTimeout = 0;
        asm("   andi    r22, 0xDF       ");
        asm("   sts     OS_Flags, r22   ");

        asm("   lds     r23, OS_state   ");
        
        asm("   tst     r24             ");     //if (bEvent)
        asm("   breq    OS_CE_BAD       ");
        {    
            #ifdef OS_ENABLE_TTIMERS
                                                    // Clear bDelay in temporary variable OS_state
                asm("   andi    r23, 0xEF       "); //_OS_bTaskDelay = 0;
                asm("   sts     OS_state, r23   ");

                _OS_SET_FSR_CUR_TASK();
                OS_ATOMIC_WRITE_A
                (
                    asm("   ld      r25, x      ");    // Clear bDelay in task descriptor
                    asm("   andi    r25, 0xEF   ");
                    asm("   st      x, r25      "); 
                );
            #endif
            asm("   ret                     ");     // Exit OK
        }

        asm("OS_CE_BAD:                  ");

        #ifdef OS_ENABLE_TTIMERS

            
            asm("   sbrs    r23, 4          ");     //if (_OS_bTaskTimeout && _OS_bTaskCanContinue)
            asm("   sbrs    r23, 5          ");
            asm("   rjmp    OS_CE_NO_TIMEOUT");
            {
                asm("   ori     r22, 0x20       "); //OS_Flags.bTimeout = 1;
                asm("   sts     OS_Flags, r22   ");
                asm("   ret                     "); // Exit OK
            }
            asm("OS_CE_NO_TIMEOUT:              ");

        #endif

        _OS_POP();
        __OS_JMP("OS_SchedRetPointNoSave");

    //------------------------------------------------------------------------------
    #else
    //------------------------------------------------------------------------------

        asm("   lds     r22, OS_Flags   ");             // OS_Flags.bTimeout = 0;
        asm("   andi    r22, 0xDF       ");
        asm("   sts     OS_Flags, r22   ");

        asm("   lds     r23, OS_state   ");

        asm("   tst     r24             ");             //if (bEvent)
        asm("   breq    OS_CE_BAD       ");
        {
            asm("   sbrs    r23, 3      ");             // if (_OS_bTaskReady) {
            asm("   rjmp    OS_CE_WAIT_MORE");
            {
                #ifdef OS_ENABLE_TTIMERS

                    asm("   andi    r23, 0xEF   ");     //_OS_bTaskDelay = 0;
                    _OS_SET_FSR_CUR_TASK();
                    OS_ATOMIC_WRITE_A
                    (
                        asm("   ld   r22, x         ");     // Clear bDelay in task descriptor
                        asm("   andi r22, 0xEF      ");
                        asm("   st   x, r22         ");
                    );

                #endif
                asm("OS_CE_EXIT_OK:             ");
                asm("   sts     OS_state, r23   ");
                asm("   ret                     ");     // Exit OK
            }

            asm("OS_CE_WAIT_MORE:               ");
            asm("   sbr     r23, (1<<3)         ");     // _OS_bTaskReady = 1;
            asm("   rjmp    OS_CE_CHECK_TIMEOUT ");
        }

        asm("OS_CE_BAD:                     ");
        asm("   cbr     r23, (1<<3)         ");         // _OS_bTaskReady = 0;

        asm("OS_CE_CHECK_TIMEOUT:           ");

        #ifdef OS_ENABLE_TTIMERS

            asm("   sbrs    r23, 4          ");         //if (_OS_bTaskTimeout && _OS_bTaskCanContinue)
            asm("   sbrs    r23, 5          ");
            asm("   rjmp    OS_CE_NO_TIMEOUT");
            {
                asm("   ori     r22, 0x20       ");     //OS_Flags.bTimeout = 1;
                asm("   sts     OS_Flags, r22   ");
                asm("   sbr     r23, (1<<3)     ");     // _OS_bTaskReady = 1;         // *91103.3*
                asm("   rjmp    OS_CE_EXIT_OK   ");     // Exit OK
            }
            asm("OS_CE_NO_TIMEOUT:              ");
        #endif

        asm("   sts     OS_state, r23       ");
        _OS_POP();
        __OS_JMP("OS_SchedRetPointNoSave");

    //------------------------------------------------------------------------------
    #endif
    //------------------------------------------------------------------------------

}








